@@ -21,8 +21,11 @@ from ironic_lib import mdns
 from oslo_config import cfg
 from oslo_log import log
 import oslo_messaging as messaging
+from oslo_utils import excutils
 from oslo_utils import reflection
+import tooz
 
+from ironic_inspector.common import coordination
 from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
 from ironic_inspector.common import keystone
@@ -36,12 +39,12 @@ from ironic_inspector import utils
 
 LOG = log.getLogger(__name__)
 CONF = cfg.CONF
-MANAGER_TOPIC = 'ironic-inspector-conductor'
+MANAGER_TOPIC = 'ironic_inspector.conductor'
 
 
 class ConductorManager(object):
     """ironic inspector conductor manager"""
-    RPC_API_VERSION = '1.2'
+    RPC_API_VERSION = '1.3'
 
     target = messaging.Target(version=RPC_API_VERSION)
 
@@ -62,9 +65,9 @@ class ConductorManager(object):
             LOG.warning('Introspection data will not be stored. Change '
                         '"[processing] store_data" option if this is not '
                         'the desired behavior')
-        elif CONF.processing.store_data == 'swift':
-            LOG.info('Introspection data will be stored in Swift in the '
-                     'container %s', CONF.swift.container)
+        else:
+            LOG.info('Introspection data will be stored in the %s backend',
+                     CONF.processing.store_data)
 
         db.init()
 
@@ -95,7 +98,30 @@ class ConductorManager(object):
             self._zeroconf.register_service('baremetal-introspection',
                                             endpoint)
 
+        if not CONF.standalone:
+            try:
+                coordinator = coordination.get_coordinator(prefix='conductor')
+                coordinator.start(heartbeat=True)
+                coordinator.join_group()
+            except tooz.ToozError:
+                with excutils.save_and_reraise_exception():
+                    LOG.critical('Failed when connecting to coordination '
+                                 'backend.')
+                    self.del_host()
+            else:
+                LOG.info('Successfully connected to coordination backend.')
+
     def del_host(self):
+        """Shutdown the ironic inspector conductor service."""
+
+        if not CONF.standalone:
+            try:
+                coordinator = coordination.get_coordinator(prefix='conductor')
+                if coordinator.started:
+                    coordinator.leave_group()
+                    coordinator.stop()
+            except tooz.ToozError:
+                LOG.exception('Failed to stop coordinator')
 
         if not self._shutting_down.acquire(blocking=False):
             LOG.warning('Attempted to shut down while already shutting down')
@@ -119,6 +145,7 @@ class ConductorManager(object):
             self._zeroconf = None
 
         self._shutting_down.release()
+
         LOG.info('Shut down successfully')
 
     def _periodics_watchdog(self, callable_, activity, spacing, exc_info,
@@ -155,6 +182,10 @@ class ConductorManager(object):
 
         process.reapply(node_uuid, data=data)
 
+    @messaging.expected_exceptions(utils.Error)
+    def do_continue(self, context, data):
+        return process.process(data)
+
 
 def periodic_clean_up():  # pragma: no cover
     try:
