@@ -15,7 +15,6 @@
 
 from oslo_config import cfg
 
-from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
 from ironic_inspector import node_cache
 from ironic_inspector import utils
@@ -47,14 +46,8 @@ def _extract_node_driver_info(introspection_data):
 def _check_existing_nodes(introspection_data, node_driver_info, ironic):
     macs = utils.get_valid_macs(introspection_data)
     if macs:
-        # verify existing ports
-        for mac in macs:
-            ports = ironic.port.list(address=mac)
-            if not ports:
-                continue
-            raise utils.Error(
-                _('Port %(mac)s already exists, uuid: %(uuid)s') %
-                {'mac': mac, 'uuid': ports[0].uuid}, data=introspection_data)
+        ir_utils.lookup_node_by_macs(macs, introspection_data, ironic=ironic,
+                                     fail=True)
     else:
         LOG.warning('No suitable interfaces found for discovered node. '
                     'Check that validate_interfaces hook is listed in '
@@ -63,17 +56,9 @@ def _check_existing_nodes(introspection_data, node_driver_info, ironic):
     # verify existing node with discovered ipmi address
     ipmi_address = node_driver_info.get('ipmi_address')
     if ipmi_address:
-        # FIXME(aarefiev): it's not effective to fetch all nodes, and may
-        #                  impact on performance on big clusters
-        nodes = ironic.node.list(fields=('uuid', 'driver_info'), limit=0)
-        for node in nodes:
-            bmc_address, bmc_ipv4, bmc_ipv6 = ir_utils.get_ipmi_address(node)
-            if ipmi_address in (bmc_ipv4, bmc_ipv6):
-                raise utils.Error(
-                    _('Node %(uuid)s already has BMC address '
-                      '%(ipmi_address)s, not enrolling') %
-                    {'ipmi_address': ipmi_address, 'uuid': node.uuid},
-                    data=introspection_data)
+        ir_utils.lookup_node_by_bmc_addresses([ipmi_address],
+                                              introspection_data,
+                                              ironic=ironic, fail=True)
 
 
 def enroll_node_not_found_hook(introspection_data, **kwargs):
