@@ -13,7 +13,6 @@
 
 """Standard set of plugins."""
 
-
 from ironic_lib import utils as il_utils
 import netaddr
 from oslo_config import cfg
@@ -27,7 +26,6 @@ from ironic_inspector import utils
 
 CONF = cfg.CONF
 
-
 LOG = utils.getProcessingLogger('ironic_inspector.plugins.standard')
 
 
@@ -102,39 +100,33 @@ class SchedulerHook(base.ProcessingHook):
         """Update node with scheduler properties."""
         inventory = utils.get_inventory(introspection_data,
                                         node_info=node_info)
-        errors = []
-
         try:
             introspection_data['cpus'] = int(inventory['cpu']['count'])
             introspection_data['cpu_arch'] = six.text_type(
                 inventory['cpu']['architecture'])
         except (KeyError, ValueError, TypeError):
-            errors.append(_('malformed or missing CPU information: %s') %
-                          inventory.get('cpu'))
+            LOG.warning('malformed or missing CPU information: %s',
+                        inventory.get('cpu'))
 
         try:
             introspection_data['memory_mb'] = int(
                 inventory['memory']['physical_mb'])
         except (KeyError, ValueError, TypeError):
-            errors.append(_('malformed or missing memory information: %s; '
-                            'introspection requires physical memory size '
-                            'from dmidecode') % inventory.get('memory'))
+            LOG.warning('malformed or missing memory information: %s; '
+                        'introspection requires physical memory size '
+                        'from dmidecode', inventory.get('memory'))
 
-        if errors:
-            raise utils.Error(_('The following problems encountered: %s') %
-                              '; '.join(errors),
-                              node_info=node_info, data=introspection_data)
-
-        LOG.info('Discovered data: CPUs: %(cpus)s %(cpu_arch)s, '
-                 'memory %(memory_mb)s MiB',
+        LOG.info('Discovered data: CPUs: count %(cpus)s, architecture '
+                 '%(cpu_arch)s, memory %(memory_mb)s MiB',
                  {key: introspection_data.get(key) for key in self.KEYS},
                  node_info=node_info, data=introspection_data)
 
         overwrite = CONF.processing.overwrite_existing
         properties = {key: str(introspection_data[key])
-                      for key in self.KEYS if overwrite or
-                      not node_info.node().properties.get(key)}
-        node_info.update_properties(**properties)
+                      for key in self.KEYS if introspection_data.get(key) and
+                      (overwrite or not node_info.node().properties.get(key))}
+        if properties:
+            node_info.update_properties(**properties)
 
 
 class ValidateInterfacesHook(base.ProcessingHook):
@@ -162,7 +154,13 @@ class ValidateInterfacesHook(base.ProcessingHook):
         for iface in inventory['interfaces']:
             name = iface.get('name')
             mac = iface.get('mac_address')
-            ip = iface.get('ipv4_address')
+            ipv4_address = iface.get('ipv4_address')
+            ipv6_address = iface.get('ipv6_address')
+            # NOTE(kaifeng) ipv6 address may in the form of fd00::1%enp2s0,
+            # which is not supported by netaddr, remove the suffix if exists.
+            if ipv6_address and '%' in ipv6_address:
+                ipv6_address = ipv6_address.split('%')[0]
+            ip = ipv4_address or ipv6_address
             client_id = iface.get('client_id')
 
             if not name:
@@ -221,7 +219,8 @@ class ValidateInterfacesHook(base.ProcessingHook):
                 LOG.debug('Skipping interface %s as it was not PXE booting',
                           name, data=data)
                 continue
-            elif CONF.processing.add_ports != 'all' and not ip:
+            elif CONF.processing.add_ports != 'all' and (
+                        not ip or netaddr.IPAddress(ip).is_link_local()):
                 LOG.debug('Skipping interface %s as it did not have '
                           'an IP address assigned during the ramdisk run',
                           name, data=data)
@@ -238,9 +237,11 @@ class ValidateInterfacesHook(base.ProcessingHook):
         """Validate information about network interfaces."""
 
         bmc_address = utils.get_ipmi_address_from_data(introspection_data)
+        bmc_v6address = utils.get_ipmi_v6address_from_data(introspection_data)
         # Overwrite the old ipmi_address field to avoid inconsistency
         introspection_data['ipmi_address'] = bmc_address
-        if not bmc_address:
+        introspection_data['ipmi_v6address'] = bmc_v6address
+        if not (bmc_address or bmc_v6address):
             LOG.debug('No BMC address provided in introspection data, '
                       'assuming virtual environment', data=introspection_data)
 
