@@ -15,7 +15,7 @@ import contextlib
 import os
 import re
 
-from eventlet.green import subprocess
+from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log
 
@@ -50,8 +50,18 @@ class IptablesFilter(pxe_filter.BaseFilter):
         self.interface = CONF.iptables.dnsmasq_interface
         self.chain = CONF.iptables.firewall_chain
         self.new_chain = self.chain + '_temp'
+
+        # Determine arguments used for pxe filtering, we only support 4 and 6
+        # at this time.
+        if CONF.iptables.ip_version == '4':
+            self._cmd_iptables = 'iptables'
+            self._dhcp_port = '67'
+        else:
+            self._cmd_iptables = 'ip6tables'
+            self._dhcp_port = '547'
+
         self.base_command = ('sudo', 'ironic-inspector-rootwrap',
-                             CONF.rootwrap_config, 'iptables')
+                             CONF.rootwrap_config, self._cmd_iptables)
 
     def reset(self):
         self.enabled = True
@@ -68,10 +78,9 @@ class IptablesFilter(pxe_filter.BaseFilter):
         # -w flag makes iptables wait for xtables lock, but it's not supported
         # everywhere yet
         try:
-            with open(os.devnull, 'wb') as null:
-                subprocess.check_call(self.base_command + ('-w', '-h'),
-                                      stderr=null, stdout=null)
-        except subprocess.CalledProcessError:
+            cmd = self.base_command + ('-w', '-h')
+            processutils.execute(*cmd)
+        except processutils.ProcessExecutionError:
             LOG.warning('iptables does not support -w flag, please update '
                         'it to at least version 1.4.21')
         else:
@@ -138,9 +147,9 @@ class IptablesFilter(pxe_filter.BaseFilter):
 
         # Swap chains
         self._iptables('-I', 'INPUT', '-i', self.interface, '-p', 'udp',
-                       '--dport', '67', '-j', chain)
+                       '--dport', self._dhcp_port, '-j', chain)
         self._iptables('-D', 'INPUT', '-i', self.interface, '-p', 'udp',
-                       '--dport', '67', '-j', main_chain,
+                       '--dport', self._dhcp_port, '-j', main_chain,
                        ignore=True)
         self._iptables('-F', main_chain, ignore=True)
         self._iptables('-X', main_chain, ignore=True)
@@ -151,23 +160,20 @@ class IptablesFilter(pxe_filter.BaseFilter):
         cmd = self.base_command + args
         ignore = kwargs.pop('ignore', False)
         LOG.debug('Running iptables %s', args)
-        kwargs['stderr'] = subprocess.STDOUT
         try:
-            subprocess.check_output(cmd, **kwargs)
-        except subprocess.CalledProcessError as exc:
-            decoded_output = exc.output.decode("utf-8")
-            output = decoded_output.replace('\n', '. ')
+            processutils.execute(*cmd)
+        except processutils.ProcessExecutionError as exc:
             if ignore:
-                LOG.debug('Ignoring failed iptables %(args)s: %(output)s',
-                          {'args': args, 'output': output})
+                LOG.debug('Ignoring failed iptables %(args)s: %(error)s',
+                          {'args': args, 'error': exc})
             else:
-                LOG.error('iptables %(iptables)s failed: %(exc)s',
-                          {'iptables': args, 'exc': output})
+                LOG.error('iptables %(iptables)s failed: %(error)s',
+                          {'iptables': args, 'error': exc})
                 raise
 
     def _clean_up(self, chain):
         self._iptables('-D', 'INPUT', '-i', self.interface, '-p', 'udp',
-                       '--dport', '67', '-j', chain,
+                       '--dport', self._dhcp_port, '-j', chain,
                        ignore=True)
         self._iptables('-F', chain, ignore=True)
         self._iptables('-X', chain, ignore=True)
