@@ -66,7 +66,37 @@ class TestIntrospect(BaseTest):
         cli.node.validate.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with('mac',
+                                                             self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+                                                         'pxe',
+                                                         persistent=False)
+        cli.node.set_power_state.assert_called_once_with(self.uuid,
+                                                         'reboot')
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
+
+    @mock.patch.object(ir_utils, 'get_ipmi_address', autospec=True)
+    def test_resolved_bmc_address(self, ipmi_mock, client_mock, start_mock):
+        self.node.driver_info['ipmi_address'] = 'example.com'
+        addresses = ['93.184.216.34', '2606:2800:220:1:248:1893:25c8:1946']
+        ipmi_mock.return_value = ('example.com',) + tuple(addresses)
+        cli = self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid)
+
+        cli.node.get.assert_called_once_with(self.uuid)
+        cli.node.validate.assert_called_once_with(self.uuid)
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=addresses,
+                                           manage_boot=True,
                                            ironic=cli)
         self.node_info.ports.assert_called_once_with()
         self.node_info.add_attribute.assert_called_once_with('mac',
@@ -91,7 +121,8 @@ class TestIntrospect(BaseTest):
         cli.node.validate.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=None,
+                                           bmc_address=[],
+                                           manage_boot=True,
                                            ironic=cli)
         self.node_info.ports.assert_called_once_with()
         self.node_info.add_attribute.assert_called_once_with('mac',
@@ -114,12 +145,12 @@ class TestIntrospect(BaseTest):
             introspect.introspect(self.node.uuid)
 
         start_mock.assert_called_with(self.uuid,
-                                      bmc_address=self.bmc_address,
+                                      bmc_address=[self.bmc_address],
+                                      manage_boot=True,
                                       ironic=cli)
 
     def test_power_failure(self, client_mock, start_mock):
         cli = self._prepare(client_mock)
-        cli.node.set_boot_device.side_effect = exceptions.BadRequest()
         cli.node.set_power_state.side_effect = exceptions.BadRequest()
         start_mock.return_value = self.node_info
 
@@ -128,7 +159,8 @@ class TestIntrospect(BaseTest):
         cli.node.get.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
                                            ironic=cli)
         cli.node.set_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
@@ -150,7 +182,8 @@ class TestIntrospect(BaseTest):
         cli.node.get.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
                                            ironic=cli)
         self.assertFalse(cli.node.set_boot_device.called)
         start_mock.return_value.finished.assert_called_once_with(
@@ -158,6 +191,28 @@ class TestIntrospect(BaseTest):
         self.node_info.acquire_lock.assert_called_once_with()
         self.node_info.release_lock.assert_called_once_with()
 
+    def test_set_boot_device_failure(self, client_mock, start_mock):
+        cli = self._prepare(client_mock)
+        cli.node.set_boot_device.side_effect = exceptions.BadRequest()
+        start_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid)
+
+        cli.node.get.assert_called_once_with(self.uuid)
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+                                                         'pxe',
+                                                         persistent=False)
+        cli.node.set_power_state.assert_not_called()
+        start_mock.return_value.finished.assert_called_once_with(
+            introspect.istate.Events.error, error=mock.ANY)
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
+
     def test_no_macs(self, client_mock, start_mock):
         cli = self._prepare(client_mock)
         self.node_info.ports.return_value = []
@@ -168,7 +223,8 @@ class TestIntrospect(BaseTest):
         self.node_info.ports.assert_called_once_with()
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
                                            ironic=cli)
         self.assertFalse(self.node_info.add_attribute.called)
         self.assertFalse(self.sync_filter_mock.called)
@@ -315,6 +371,27 @@ class TestIntrospect(BaseTest):
         # updated to the current time.time()
         self.assertEqual(100, introspect._LAST_INTROSPECTION_TIME)
 
+    def test_no_manage_boot(self, client_mock, add_mock):
+        cli = self._prepare(client_mock)
+        self.node_info.manage_boot = False
+        add_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid, manage_boot=False)
+
+        cli.node.get.assert_called_once_with(self.uuid)
+
+        add_mock.assert_called_once_with(self.uuid,
+                                         bmc_address=[self.bmc_address],
+                                         manage_boot=False,
+                                         ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with('mac',
+                                                             self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        self.assertFalse(cli.node.validate.called)
+        self.assertFalse(cli.node.set_boot_device.called)
+        self.assertFalse(cli.node.set_power_state.called)
+
 
 @mock.patch.object(node_cache, 'get_node', autospec=True)
 @mock.patch.object(ir_utils, 'get_client', autospec=True)
@@ -337,8 +414,7 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.node.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
         cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
@@ -346,6 +422,24 @@ class TestAbort(BaseTest):
             introspect.istate.Events.abort_end, error='Canceled by operator')
         self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
 
+    def test_no_manage_boot(self, client_mock, get_mock):
+        cli = self._prepare(client_mock)
+        get_mock.return_value = self.node_info
+        self.node_info.acquire_lock.return_value = True
+        self.node_info.started_at = time.time()
+        self.node_info.finished_at = None
+        self.node_info.manage_boot = False
+
+        introspect.abort(self.node.uuid)
+
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
+        self.node_info.acquire_lock.assert_called_once_with(blocking=False)
+        self.sync_filter_mock.assert_called_once_with(cli)
+        self.assertFalse(cli.node.set_power_state.called)
+        self.node_info.finished.assert_called_once_with(
+            introspect.istate.Events.abort_end, error='Canceled by operator')
+        self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
+
     def test_node_not_found(self, client_mock, get_mock):
         cli = self._prepare(client_mock)
         exc = utils.Error('Not found.', code=404)
@@ -383,8 +477,7 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
         cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
@@ -402,8 +495,7 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
         cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
