@@ -161,6 +161,13 @@ class IntrospectionDataNotFound(NotFoundInCacheError):
     """Introspection data not found."""
 
 
+class NoAvailableConductor(Error):
+    """No available conductor in the service group."""
+
+    def __init__(self, msg, **kwargs):
+        super(NoAvailableConductor, self).__init__(msg, code=503, **kwargs)
+
+
 def executor():
     """Return the current futures executor."""
     global _EXECUTOR
